class JSDisposableStack;
// Alias for IsJSDisposableStack() that avoids inlining.
// https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=12&c=1
V8_EXPORT_PRIVATE bool IsJSDisposableStack_NonInline(Tagged<HeapObject> o);

// Definition https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=12&c=1
template <class D, class P>
class TorqueGeneratedJSDisposableStack : public P {
  static_assert(
      std::is_same<JSDisposableStack, D>::value,
      "Use this class as direct base for JSDisposableStack.");
  static_assert(
      std::is_same<JSObject, P>::value,
      "Pass in JSObject as second template parameter for TorqueGeneratedJSDisposableStack.");

 public: 
  using Super = P;
  using TorqueGeneratedClass = TorqueGeneratedJSDisposableStack<D,P>;

  // https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=15&c=3
  inline Tagged<FixedArray> stack() const;
  // https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=15&c=3
  inline Tagged<FixedArray> stack(PtrComprCageBase cage_base) const;
  // https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=15&c=3
  inline void set_stack(Tagged<FixedArray> value, WriteBarrierMode mode = UPDATE_WRITE_BARRIER);

  // https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=16&c=3
  inline int status() const;
  // https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=16&c=3
  inline void set_status(int value);

  // https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=12&c=1
  V8_INLINE static Tagged<D> cast(Tagged<Object> object);
  // https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=12&c=1
  V8_INLINE static Tagged<D> unchecked_cast(Tagged<Object> object) {
    return Tagged<D>::unchecked_cast(object);
  }

  DECL_PRINTER(JSDisposableStack)

#ifdef VERIFY_HEAP
  // https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=12&c=1
  V8_EXPORT_PRIVATE void JSDisposableStackVerify(Isolate* isolate);
#endif  // VERIFY_HEAP

  static constexpr int kStartOfStrongFieldsOffset = P::kHeaderSize;
  // https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=15&c=3
  static constexpr int kStackOffset = P::kHeaderSize;
  static constexpr int kStackOffsetEnd = kStackOffset + kTaggedSize - 1;
  // https://source.chromium.org/chromium/chromium/src/+/main:v8/src/objects/js-disposable-stack.tq?l=16&c=3
  static constexpr int kStatusOffset = kStackOffsetEnd + 1;
  static constexpr int kStatusOffsetEnd = kStatusOffset + kTaggedSize - 1;
  static constexpr int kEndOfStrongFieldsOffset = kStatusOffsetEnd + 1;
  static constexpr int kStartOfWeakFieldsOffset = kStatusOffsetEnd + 1;
  static constexpr int kEndOfWeakFieldsOffset = kStatusOffsetEnd + 1;
  static constexpr int kHeaderSize = kStatusOffsetEnd + 1;

  friend class Factory;

  template <class DAlias = D>
  constexpr TorqueGeneratedJSDisposableStack() : P() {
    static_assert(
        std::is_base_of<TorqueGeneratedJSDisposableStack, DAlias>::value,
        "class TorqueGeneratedJSDisposableStack should be used as direct base for JSDisposableStack.");
  }

 protected:
  inline explicit constexpr TorqueGeneratedJSDisposableStack(Address ptr, typename P::SkipTypeCheckTag
)    : P(ptr, typename P::SkipTypeCheckTag{}) {}
  inline explicit TorqueGeneratedJSDisposableStack(Address ptr);
};

